package org.japura.controller;

import java.awt.Component;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;

import org.japura.Application;
import org.japura.debug.DebugComponent;
import org.japura.modal.Modal;

/**
 * <P>
 * Copyright (C) 2013 Carlos Eduardo Leite de Andrade
 * <P>
 * This library is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * <P>
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * <P>
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <A
 * HREF="www.gnu.org/licenses/">www.gnu.org/licenses/</A>
 * <P>
 * For more information, contact: <A HREF="www.japura.org">www.japura.org</A>
 * <P>
 * 
 * @author Carlos Eduardo Leite de Andrade
 */
public class DefaultControllerManager extends AbstractControllerManager{

  @Override
  public void closeModal(ControllerModel model) {
	Controller<?> controller = (Controller<?>) model;
	if (controller.isComponentInstancied()) {
	  Component component = controller.getComponent();
	  if (component != null) {
		if (component instanceof JFrame) {
		  JFrame frame = (JFrame) component;
		  Modal.closeAllModals(frame);
		} else if (component instanceof JDialog) {
		  JDialog dialog = (JDialog) component;
		  Modal.closeAllModals(dialog);
		} else if (component instanceof JInternalFrame) {
		  JInternalFrame frame = (JInternalFrame) component;
		  Modal.closeAllModals(frame);
		}
	  }
	}
  }

  @Override
  public void removeFromDebugComponents(ControllerModel model) {
	Controller<?> controller = (Controller<?>) model;
	if (DebugComponent.exists(DebugControllers.class)) {
	  DebugControllers.Parameter parameter =
		  new DebugControllers.Parameter(controller);
	  DebugComponent.publish(DebugControllers.class, parameter);
	}
  }

  @Override
  public void updateDebugComponents() {
	if (DebugComponent.exists(DebugControllers.class)) {
	  DebugControllers.Parameter parameter = new DebugControllers.Parameter();
	  DebugComponent.publish(DebugControllers.class, parameter);
	}
  }

  @Override
  public Class<?> getNewInstanceClass() {
	return Controller.class;
  }

  @Override
  public final void remove(ControllerModel controller,
						   boolean cancelTaskExecution) {
	if (controller != null && controller.isPermanent() == false) {
	  List<ControllerModel> children = new ArrayList<ControllerModel>();
	  fetchHierarchy(controller, children);

	  AbstractController c = (AbstractController) controller;

	  c.beforeFreeController();
	  for (ControllerModel child : children) {
		Controller<?> cc = (Controller<?>) child;
		cc.beforeFreeController();
	  }

	  String groupId = controller.getGroupId();
	  if (cancelTaskExecution) {
		Application.getTaskManager().cancel(groupId);
	  }

	  closeModal(controller);
	  for (ControllerModel child : children) {
		closeModal(child);
	  }

	  AbstractController parentController =
		  (AbstractController) controller.getParent();
	  if (parentController != null) {
		parentController.removeChild(c);
	  }

	  c.clearParent();
	  c.clearChildren();
	  for (ControllerModel child : children) {
		Controller<?> cc = (Controller<?>) child;
		cc.clearParent();
		cc.clearChildren();
	  }

	  controllers.remove(controller);
	  c.setAsRemoved();
	  unlink(controller);
	  removeFromDebugComponents(c);
	  for (ControllerModel child : children) {
		Controller<?> cc = (Controller<?>) child;
		controllers.remove(child);
		cc.setAsRemoved();
		unlink(child);
		removeFromDebugComponents(cc);
	  }

	  c.afterFreeController();
	  for (ControllerModel child : children) {
		Controller<?> cc = (Controller<?>) child;
		cc.afterFreeController();
	  }

	  updateDebugComponents();
	  synchronized (listeners) {
		for (ControllerManagerListener listener : listeners) {
		  listener.removed(controller);
		}
	  }
	}
  }
}
