package org.japura.dialogs;

import java.awt.Component;
import java.awt.Image;
import java.awt.Window;
import java.util.ArrayList;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.SwingUtilities;

import org.japura.gui.Images;
import org.japura.i18n.I18nManager;
import org.japura.i18n.I18nStringKeys;

/**
 * <P>
 * Copyright (C) 2012 Carlos Eduardo Leite de Andrade
 * <P>
 * This library is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * <P>
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * <P>
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <A
 * HREF="www.gnu.org/licenses/">www.gnu.org/licenses/</A>
 * <P>
 * For more information, contact: <A HREF="www.japura.org">www.japura.org</A>
 * <P>
 * 
 * @author Carlos Eduardo Leite de Andrade
 */
public class DefaultMessagesDialogs implements MessagesDialogs{

  public DefaultMessageDialog buildDialog(Component owner, String title,
										  Image dialogIcon, Image icon,
										  String msg,
										  List<MessageDialogButton> buttons) {
	return buildDialog(owner, title, dialogIcon, icon, msg, null, buttons);
  }

  public DefaultMessageDialog buildDialog(Component owner, String title,
										  Image dialogIcon, Image icon,
										  String msg, String detailedMsg,
										  List<MessageDialogButton> buttons) {
	Window window = null;
	if (owner != null) {
	  window = SwingUtilities.getWindowAncestor(owner);
	}
	return new DefaultMessageDialog(window, title, dialogIcon, icon, msg,
		detailedMsg, buttons);
  }

  @Override
  public Boolean showDialog(MessageType messageType, PromptType promptType,
							Component owner, String title, String msg,
							String detailedMsg, String... buttonsTexts) {
	if (buttonsTexts == null) {
	  throw new RuntimeException("No buttonsTexts");
	}

	if (promptType.getButtonsCount() != buttonsTexts.length) {
	  throw new RuntimeException(
		  "Amount of text buttons incompatible with PromptType");
	}

	Image icon = getIcon(messageType);
	Image dialogIcon = icon;

	List<MessageDialogButton> buttons = new ArrayList<MessageDialogButton>();
	if (promptType.equals(PromptType.YES)) {
	  buttons.add(new MessageDialogButton(buttonsTexts[0], new Boolean(true)));
	} else if (promptType.equals(PromptType.YES_NO)) {
	  buttons.add(new MessageDialogButton(buttonsTexts[0], new Boolean(true)));
	  buttons.add(new MessageDialogButton(buttonsTexts[1], new Boolean(false)));
	} else if (promptType.equals(PromptType.YES_NO_CANCEL)) {
	  buttons.add(new MessageDialogButton(buttonsTexts[0], new Boolean(true)));
	  buttons.add(new MessageDialogButton(buttonsTexts[1], new Boolean(false)));
	  buttons.add(new MessageDialogButton(buttonsTexts[2], null));
	}

	DefaultMessageDialog dialog =
		buildDialog(owner, title, dialogIcon, icon, msg, detailedMsg, buttons);

	dialog.setVisible(true);

	return (Boolean) dialog.getResult();
  }

  private Image getIcon(MessageType messageType) {
	if (messageType.equals(MessageType.ERROR)) {
	  return new ImageIcon(Images.ERROR).getImage();
	} else if (messageType.equals(MessageType.INFO)) {
	  return new ImageIcon(Images.INFORMATION).getImage();
	} else if (messageType.equals(MessageType.QUESTION)) {
	  return new ImageIcon(Images.QUESTION).getImage();
	} else if (messageType.equals(MessageType.WARN)) {
	  return new ImageIcon(Images.WARNING).getImage();
	}
	return null;
  }

  @Override
  public Boolean showDialog(MessageType messageType, PromptType promptType,
							Component owner, String title, String msg,
							String... buttonsTexts) {
	return showDialog(messageType, promptType, owner, title, msg, null,
		buttonsTexts);
  }

  @Override
  public Boolean showQuestionDialog(Component owner, String title, String msg) {
	String yes = I18nManager.getString(I18nStringKeys.YES.getKey());
	String no = I18nManager.getString(I18nStringKeys.NO.getKey());
	return showDialog(MessageType.QUESTION, PromptType.YES_NO, owner, title,
		msg, null, new String[] { yes, no });
  }

  @Override
  public Boolean showQuestionDialog(Component owner, String title, String msg,
									String detailedMsg) {
	String yes = I18nManager.getString(I18nStringKeys.YES.getKey());
	String no = I18nManager.getString(I18nStringKeys.NO.getKey());
	return showDialog(MessageType.QUESTION, PromptType.YES_NO, owner, title,
		msg, detailedMsg, new String[] { yes, no });
  }

  @Override
  public Boolean showQuestionDialog(Component owner, String title, String msg,
									String detailedMsg, String yesButtonText,
									String noButtonText) {
	return showDialog(MessageType.QUESTION, PromptType.YES_NO, owner, title,
		msg, detailedMsg, new String[] { yesButtonText, noButtonText });
  }

  @Override
  public Boolean showWarnDialog(Component owner, String title, String msg,
								String detailedMsg) {
	String close = I18nManager.getString(I18nStringKeys.CLOSE.getKey());
	return showDialog(MessageType.WARN, PromptType.YES, owner, title, msg,
		detailedMsg, new String[] { close });
  }

  @Override
  public Boolean showWarnDialog(Component owner, String title, String msg) {
	String close = I18nManager.getString(I18nStringKeys.CLOSE.getKey());
	return showDialog(MessageType.WARN, PromptType.YES, owner, title, msg,
		null, new String[] { close });
  }

  @Override
  public Boolean showErrorDialog(Component owner, String title, String msg,
								 String detailedMsg) {
	String close = I18nManager.getString(I18nStringKeys.CLOSE.getKey());
	return showDialog(MessageType.ERROR, PromptType.YES, owner, title, msg,
		detailedMsg, new String[] { close });
  }

  @Override
  public Boolean showErrorDialog(Component owner, String title, String msg) {
	String close = I18nManager.getString(I18nStringKeys.CLOSE.getKey());
	return showDialog(MessageType.ERROR, PromptType.YES, owner, title, msg,
		null, new String[] { close });
  }

  @Override
  public Boolean showInfoDialog(Component owner, String title, String msg,
								String detailedMsg) {
	String close = I18nManager.getString(I18nStringKeys.CLOSE.getKey());
	return showDialog(MessageType.INFO, PromptType.YES, owner, title, msg,
		detailedMsg, new String[] { close });
  }

  @Override
  public Boolean showInfoDialog(Component owner, String title, String msg) {
	String close = I18nManager.getString(I18nStringKeys.CLOSE.getKey());
	return showDialog(MessageType.INFO, PromptType.YES, owner, title, msg,
		null, new String[] { close });
  }

}
