package org.japura.task;

import org.japura.controller.Controller;
import org.japura.controller.ControllerModel;
import org.japura.debug.DebugResult;

/**
 * <P>
 * Copyright (C) 2011-2012 Carlos Eduardo Leite de Andrade
 * <P>
 * This library is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * <P>
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * <P>
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <A
 * HREF="www.gnu.org/licenses/">www.gnu.org/licenses/</A>
 * <P>
 * For more information, contact: <A HREF="www.japura.org">www.japura.org</A>
 * <P>
 * 
 * @author Carlos Eduardo Leite de Andrade
 */
class TaskDebugLog extends DebugResult{
  private String id;
  private String name;
  private String source;
  private TaskEvent event;
  private long time;
  private boolean edt;
  private boolean waitForEDT;
  private int queueTasks;
  private long timeSpent;
  private String session;

  public TaskDebugLog(Task<?> task, TaskEvent event, long time, boolean edt,
	  int queueTasks) {
	this.id = task.getId();
	this.queueTasks = queueTasks;
	this.name = task.getName();
	this.time = time;
	this.event = event;
	this.edt = edt;
	this.waitForEDT = task.isWaitForEDT();
	this.timeSpent = task.getBackgroundTimeSpent();
	ControllerModel controller = task.getRootModel();
	if (controller == null) {
	  source = "TaskManager";
	} else {
	  if (controller instanceof Controller) {
		source =
			controller.getClass().getSimpleName() + " ["
				+ ((Controller<?>) controller).stringToDebugComponent() + "]";
	  } else {
		source = controller.getClass().getSimpleName();
	  }
	}
	TaskSession session = task.getSession();
	this.session = session.getClass().getName() + " [" + session.getId() + "]";
  }

  public int getQueueTasks() {
	return queueTasks;
  }

  public String getSource() {
	return source;
  }

  public boolean isEdt() {
	return edt;
  }

  public String getName() {
	return name;
  }

  public String getId() {
	return id;
  }

  public TaskEvent getEvent() {
	return event;
  }

  public boolean isWaitForEDT() {
	return waitForEDT;
  }

  public long getStartTime() {
	return time;
  }

  public void setTimeSpent(long timeSpent) {
	this.timeSpent = timeSpent;
  }

  public String getTimeSpent() {
	if (timeSpent < 1000) {
	  return Long.toString(timeSpent) + "ms";
	}

	long ms = timeSpent % 1000;
	long s = timeSpent / 1000;
	long m = timeSpent / 60000;
	long h = timeSpent / 3600000;

	StringBuilder sb = new StringBuilder();
	if (h > 0) {
	  sb.append(h);
	  sb.append("h");
	}
	if (m > 0) {
	  if (sb.length() > 0) {
		sb.append(" ");
	  }
	  sb.append(m);
	  sb.append("m");
	}
	if (s > 0) {
	  if (sb.length() > 0) {
		sb.append(" ");
	  }
	  sb.append(s);
	  sb.append("s");
	}
	if (ms > 0) {
	  if (sb.length() > 0) {
		sb.append(" ");
	  }
	  sb.append(ms);
	  sb.append("ms");
	}

	return sb.toString();
  }

  public String getSession() {
	return session;
  }
}
