package org.japura.util.worker;

import org.japura.gui.EnhancedPanel;
import org.japura.gui.ExecutionPanel;

/**
 * <P>
 * Copyright (C) 2012 Carlos Eduardo Leite de Andrade
 * <P>
 * This library is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * <P>
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * <P>
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <A
 * HREF="www.gnu.org/licenses/">www.gnu.org/licenses/</A>
 * <P>
 * For more information, contact: <A HREF="www.japura.org">www.japura.org</A>
 * <P>
 * 
 * @author Carlos Eduardo Leite de Andrade
 */
public class ModalProgressBarHandler extends SwingWorkerHandler{

  private EnhancedPanel enhancedPanel;
  private ExecutionPanel executionPanel;

  public ModalProgressBarHandler(EnhancedPanel enhancedPanel) {
	this.enhancedPanel = enhancedPanel;
  }

  @Override
  protected synchronized void before(SwingWorkerHandlerEvent event) {
	if (executionPanel == null) {
	  String text = event.getName();
	  if (text == null) {
		text = "";
	  }
	  executionPanel = new ExecutionPanel(text);
	  enhancedPanel.setEventsBlocked(true);
	  enhancedPanel.setWidgetTranslucentBackground(true);
	  enhancedPanel.addWidget(executionPanel);
	  executionPanel.start();
	}
  }

  @Override
  protected synchronized void after(SwingWorkerHandlerEvent event) {
	if (event.getWorkersCount() == 0) {
	  executionPanel.stop();
	  enhancedPanel.setEventsBlocked(false);
	  enhancedPanel.setWidgetTranslucentBackground(false);
	  enhancedPanel.removeWidget(executionPanel);
	  executionPanel = null;
	}
  }

}
